#ifndef FRACTION_H
#define FRACTION_H

#include <assert.h>
#include <iostream.h>

// File: fraction.h
// ----------------
// Library defining Fraction class for calculation with fractions
//
// (c) 1998-2000 MeeSoft - Rune M. Barnkob 
//
// YOU MAY FREELY DISTRIBUTE AND USE THIS FILE - JUST CREDIT ORIGINAL AUTHOR.
//
// Contents:
// ---------
// Fraction
//  overrides: +(int), +(Fraction)
//             -(int), -(Fraction), -()
//             *(int), *(Fraction)
//             /(int), /(Fraction)
//             ++, --
//             +=(int), +=(Fraction)
//             -=(int), -=(Fraction)
//             *=(int), *=(Fraction)
//             /=(int), /=(Fraction)
//             !() : returns x^-1
//              

class Fraction {
  private:
    long nom;  // nominator
    long denom;  // denominator
  public:
    // default constructor (0/1)
    Fraction() { nom=0; denom=1; return; }

    // fraction constructor
    Fraction( long nominator, long denominator=1 ):
      nom(nominator), denom(denominator) {
      assert( denom!=0 );
      simplify();
    }

    // reduce to simplest fraction (using euclides GCD algorithm)
    void simplify() {
      int t=0, a=nom, b=denom;
		
      if ( (a==0) || (b==0) ) return;

      if (a>b) {
        t=b;
        b=a;
        a=t;
      }
      // now b is larger than a
      // a working example of Euclides algorithm
      int c=b, d=a;
      do {
        b=c;
        a=d;
        c=a;
        d=b%a;
      } while (d!=0);
      // a equals the greatest common divisor
      nom /= a;
      denom /= a;
    }

    Fraction operator +(const Fraction& b) { return Fraction( nom*b.denom+b.nom*denom, denom*b.denom ); }

    Fraction operator -(const Fraction& b) { return Fraction( nom*b.denom-b.nom*denom, denom*b.denom ); }
    Fraction operator -() { return Fraction( -nom, denom ); }
    Fraction operator *(const Fraction& b) { return Fraction( nom*b.nom, denom*b.denom ); }
    Fraction operator /(const Fraction& b) { return Fraction( nom*b.denom, denom*b.nom ); }
    Fraction operator ++() { return Fraction( nom+denom, denom ); }
    Fraction operator --() { return Fraction( nom-denom, denom ); }

    void operator +=(const Fraction& b) { nom = nom*b.denom+b.nom*denom; denom = denom*b.denom; simplify(); return ; }
    void operator -=(const Fraction& b) { nom = nom*b.denom-b.nom*denom; denom = denom*b.denom; simplify(); return ; }
    void operator *=(const Fraction& b) { nom = nom*b.nom; denom = denom*b.denom; simplify(); return ; }
    void operator /=(const Fraction& b) { nom = nom*b.denom; denom = denom*b.nom; simplify(); return ; }

    Fraction operator !() { return Fraction( denom, nom ); }

    friend ostream& operator<<(ostream&, const Fraction&);
};

ostream& operator<<(ostream& o, const Fraction& f) {
  o << f.nom;
  if (f.denom!=1) o << "/" << f.denom;
  return o;
}

#endif
