#ifndef SWITCHALG_H
#define SWITCHALG_H
//
// file: switchalg.h
// -----------------
// Class for switching-algebra (Boolsk Algebra) for use with C++.
//
// (c) 1997-2000 MeeSoft - Rune M. Barnkob
//
// YOU MAY FREELY DISTRIBUTE AND USE THIS FILE - JUST CREDIT ORIGINAL AUTHOR.
// 
// Contents:
// ---------
// SwitchAlg
//   constructor: () - init with false
//                (bool) - init with bool value
//   methods:     getState() - returns current state
//                setState(bool) - set the state to bool
//   operator:    ! - logic NOT
//                +(SwitchAlg) - logical OR
//                *(SwitchAlg) - logical AND
//                ==(SwitchAlg) - compares to SwitchAlg objects
//                <<(...,...) - friend method for outputting to stream
using namespace std;
#include <iostream.h>

class SwitchAlg {
	public:
		// default constructor initialises to false
		SwitchAlg(): state(false) {}
		
		// force value in constructor
		SwitchAlg(bool startstate): state(startstate) {}
		
		// get the state 
		bool getState() const {
			return state;
		}
		
		// set the state 
		void setState(bool newstate) {
			state = newstate;
		}
		
		// perform logical NOT and return new TSwitch with NOT-value
		SwitchAlg operator !(void) const {
			return ( SwitchAlg(!state) );
		}

		// perform logical OR between two states and return result as TSwitch
		SwitchAlg operator +(const SwitchAlg& b) const {
			return ( SwitchAlg( state || b.state ) );
		}

		// perform logical AND between two states and return result as TSwitch
		SwitchAlg operator *(const SwitchAlg& b) const {
			return ( SwitchAlg( state && b.state ) );
		}
		
		// comparison
		bool operator==(const SwitchAlg& b) const {
			return state==b.state;
		}

		friend ostream& operator<<(ostream&, const SwitchAlg&);
	
	protected:
		bool state;
};

ostream& operator<<(ostream& o, const SwitchAlg& a) {
	if (a.state) o << "1"; else o << "0";
	return o;
}

#endif