#ifndef VECTOR_H
#define VECTOR_H
#include <iostream>
class Vector3D {
 public:
 double x,y,z;


 // Construct with values - values not supplied are assumed =0
 Vector3D(double=0, double=0, double=0);

 // Destructor (default)
 ~Vector3D();

 // Get the length of the vector;
 double len();

 // Support for IO-streams
 friend ostream& operator<<(ostream& o, const Vector3D& v);

 // Add two vectors
 Vector3D inline operator+(const Vector3D& b) const;

 // Substraction of two vectors
 Vector3D inline operator-(const Vector3D& b) const;

 // Negate the vector
 Vector3D inline operator-() const;

 // Vector product
 Vector3D inline operator/(const Vector3D& b) const;

 // For proper support of *, a friend function is needed
 friend inline double operator*(const Vector3D& a, const Vector3D& b);
 friend inline Vector3D operator*(double a, const Vector3D& b);
 friend inline Vector3D operator*(const Vector3D& a, double b);
};

#define MAX(a,b) (a>b?a:b)

// Constructor
Vector3D::Vector3D( double xx=0, double yy=0, double zz=0 ) :
  x(xx), y(yy), z(zz) {}

// Destructor
Vector3D::~Vector3D() {}

// Get length of vector
double Vector3D::len() {
  return ( (double) sqrt(x*x+y*y+z*z));
}

// Support for IO-streams
ostream& operator<<(ostream& o, const Vector3D& v) {
  return (o << "(" << v.x << "," << v.y << "," << v.z << ")");
}

// Addition
inline Vector3D Vector3D::operator+(const Vector3D& b) const {
  return Vector3D(x+b.x, y+b.y, z+b.z);
}

// Subtraction
inline Vector3D Vector3D::operator-(const Vector3D& b) const {
  return Vector3D(x-b.x, y-b.y, z-b.z);
}

// Negate the vector
inline Vector3D Vector3D::operator-() const {
  return Vector3D(-x,-y,-z);
}

// Vector product (cross product)
inline Vector3D Vector3D::operator/(const Vector3D& b) const {
  return Vector3D(z*b.z-x*b.x, x*b.x-y*b.y, y*b.y-z*b.z);
}


inline double operator*(const Vector3D& a, const Vector3D& b) {
  return (a.x*b.x + a.y*b.y + a.z*b.z);
}

inline Vector3D operator*(double a, const Vector3D& b) {
  return Vector3D(a*b.x, a*b.y, a*b.z);
}

inline Vector3D operator*(const Vector3D& a, double b) {
  return Vector3D(b*a.x, b*a.y, b*a.z);
}

#endif
